{v1.1}
{COPYFROM [d:][infile] [outfile] [logfile]           }
{   Copies infile from as many floppies as necessary.}
{   Default drive for infile is A:.                  }
{   Default filename for outfile is infile.          }
{   Extension of infile will increment by one ASCII  }
{   character for each new floppy disk accessed.     }
{   e.g., MYDATA.DAT, MYDAT.DAU, MYDATA.DAV, ...     }
{   The drive for logfile is the drive of infile.    }
{   Default filename for logfile is outfile with an  }
{     extension of .LOG .                            }
{   logfile should be on the last floppy in the      }
{     copyto sequence.                               }
{   Use COPYTO to create infile.                     }

program copyfrom;
const
  buffer_size=16384;
  total_bytes:real=0.0;
var
  in_name          : string[80];
  out_name         : string[80];
  log_name         : string[80];
  in_file,out_file : file;
  log_file         : text;
  buffer           : array [1..buffer_size] of byte;
  bytes_in         : integer;
  bytes_out        : integer;
  log_bytes        : real;
  done             : boolean;
  i                : integer;

function exists(var filename_of_any_string_length):boolean;
var
  filename : string[255] absolute filename_of_any_string_length;
  filein : text;
begin
  assign(filein,filename);
  {$I-}
  reset(filein);
  {$I+}
  if ioresult = 0 then begin
    exists:=true;
    close(filein);
    end
  else exists:=false;
  end; {exists}

begin
  log_bytes:=0.0;
  total_bytes:=0.0;

  if paramstr(1) = '' then begin
    writeln;
    writeln('COPYFROM [[d:]infile [outfile [logfile]]]           ');
    writeln('   Copies infile from as many floppies as necessary.');
    writeln('   Default drive for infile is A:.                  ');
    writeln('   Default filename for outfile is infile.          ');
    writeln('   Extension of infile will increment by one ASCII  ');
    writeln('   character for each new floppy disk accessed.     ');
    writeln('   e.g., MYDATA.DAT, MYDAT.DAU, MYDATA.DAV, ...     ');
    writeln('   The drive for logfile is the drive of infile.    ');
    writeln('   Default filename for logfile is outfile with an  ');
    writeln('     extension of .LOG .                            ');
    writeln('   logfile should be on the last floppy in the      ');
    writeln('     copyto sequence.                               ');
    writeln('   Use COPYTO to create infile.                     ');
    writeln;
    halt;
    end;

  in_name:=paramstr(1);
  if pos('.',in_name) = 0 then in_name:=in_name+'. ';
  if in_name[2] <> ':' then in_name:='A:'+in_name;

  if paramstr(2) = '' then begin
    out_name:=copy(in_name,3,length(in_name)-2);
    end
  else out_name:=paramstr(2);

  if paramstr(3) = '' then begin
    log_name:=in_name;
    end
  else begin
    log_name:=paramstr(3);
    if log_name[2] <> ':' then log_name:=copy(in_name,1,2)+log_name;
    end;
  if pos('.',log_name) > 0 then log_name:=copy(log_name,1,pos('.',log_name)-1)+
                                          '.LOG';

  for i:=1 to length( in_name) do  in_name[i]:=upcase( in_name[i]);
  for i:=1 to length(out_name) do out_name[i]:=upcase(out_name[i]);
  for i:=1 to length(log_name) do log_name[i]:=upcase(log_name[i]);

  writeln;
  writeln('Place the floppy containing ',in_name,
          ' in drive ',copy(in_name,1,2));
  writeln('Press Enter when ready.');
  readln;

  assign (out_file,out_name);
  rewrite (out_file,1);

  if exists(in_name) then begin
    repeat {until exists(log_name)}
      assign (in_file,in_name);
      reset (in_file,1);
      writeln('Restoring ',out_name,' ...');
      repeat {until bytes_in=0}
        blockread (in_file,buffer,buffer_size,bytes_in);
        total_bytes:=total_bytes+bytes_in;
        if bytes_in > 0 then blockwrite (out_file,buffer,bytes_in,bytes_out);
        until bytes_in=0;
      close (in_file);
      in_name[length(in_name)]:=chr(ord(in_name[length(in_name)])+1);
      done:=exists(log_name);
      writeln;
      if not done then begin
        writeln('Replace floppy in drive ',copy(in_name,1,2),
                ' with next sequential diskette.');
        writeln('Press Enter when ready.');
        readln;
        end;
      until done;
    close (out_file);
    assign(log_file,log_name);
    reset(log_file);
    readln(log_file,log_bytes);
    if abs(log_bytes-total_bytes) > 0.5 then begin
      writeln;
      writeln('BYTE COUNT ERROR: restoration not sucessful.');
      writeln('  Bytes logged =',log_bytes:1:0,', bytes read =',total_bytes:1:0);
      end
    else begin
      writeln;
      writeln(out_name,' restored.');
      end;
    end
  else begin
    writeln(in_name,' not found.  Copy aborted.');
    end;
  end.
