{V1.0}
{COPYTO [infile] [d:][outfile] [logfile]             }
{   Copies infile to as many floppies as necessary.  }
{   Default drive for outfile is A:.                 }
{   Default filename for outfile is infile.          }
{   Extension of outfile will increment by one ASCII }
{     character for each additional floopy required. }
{     e.g., MYDATA.DAT, MYDAT.DAU, MYDATA.DAV, ...   }
{   The drive for logfile is the drive of outfile.   }
{   Default filename for logfile is outfile with an  }
{     extension of .LOG .                            }
{   logfile will be placed on the last floppy in the }
{     copyto sequence.                               }
{   Use COPYFROM to restore infile.                  }
program copy3;
const
  buffer_size=16384;
var
  in_name          : string[80];
  out_name         : string[80];
  log_name         : string[80];
  in_file,out_file : file;
  log_file         : text;
  buffer           : array [1..buffer_size] of byte;
  recs_in          : integer;
  recs_out         : integer;
  recs_not_written : integer;
  total_recs       : real;
  i                : integer;

function exists(var filename_of_any_string_length):boolean;
var
  filename : string[255] absolute filename_of_any_string_length;
  filein : text;
begin
  assign(filein,filename);
  {$I-}
  reset(filein);
  {$I+}
  if ioresult = 0 then begin
    exists:=true;
    close(filein);
    end
  else exists:=false;
  end; {exists}

begin
  total_recs:=0.0;
  recs_not_written:=0;

  if paramstr(1) = '' then begin
    writeln;
    writeln('COPYTO [infile [d:][outfile [logfile]]]             ');
    writeln('   Copies infile to as many floppies as necessary.  ');
    writeln('   Default drive for outfile is A:.                 ');
    writeln('   Default filename for outfile is infile.          ');
    writeln('   Extension of outfile will increment by one ASCII ');
    writeln('     character for each additional floopy required. ');
    writeln('     e.g., MYDATA.DAT, MYDAT.DAU, MYDATA.DAV, ...   ');
    writeln('   The drive for logfile is the drive of outfile.   ');
    writeln('   Default filename for logfile is infile with an   ');
    writeln('     extension of .LOG .                            ');
    writeln('   logfile will be placed on the last floppy in the ');
    writeln('     copyto sequence.                               ');
    writeln('   Use COPYFROM to restore infile.                  ');
    writeln;
    halt;
    end;

  in_name:=paramstr(1);

  if paramstr(2) = '' then begin
    out_name:=in_name;
    for i:=1 to length(out_name) do if out_name[i] in [':','\'] then begin
      out_name:=copy(out_name,i+1,length(out_name)-i);
      end;
    end
  else begin
    out_name:=paramstr(2);
    if (length(out_name) = 2) and (out_name[2] = ':') then begin
      out_name:=out_name+in_name;
      for i:=3 to length(out_name) do if out_name[i] in [':','\'] then begin
        out_name:=copy(out_name,1,2)+copy(out_name,i+1,length(out_name)-i);
        end;
      end;
    end;
  if out_name[2] <> ':' then out_name:='A:'+out_name;

  if paramstr(3) = '' then begin
    log_name:=in_name;
    end
  else begin
    log_name:=paramstr(3);
    end;
  for i:=1 to length(log_name) do if log_name[i] in [':','\'] then begin
    log_name:=copy(log_name,i+1,length(log_name)-i);
    end;
  log_name:=copy(out_name,1,2)+log_name;
  if pos('.',log_name) > 0 then log_name:=copy(log_name,1,pos('.',log_name)-1)+
                                          '.LOG';

  for i:=1 to length( in_name) do  in_name[i]:=upcase( in_name[i]);
  for i:=1 to length(out_name) do out_name[i]:=upcase(out_name[i]);
  for i:=1 to length(log_name) do log_name[i]:=upcase(log_name[i]);

  if not exists(in_name) then begin
    writeln;
    writeln('ERROR: ',in_name,' does not exist.  Program terminating.');
    halt;
    end;
  assign (in_file,in_name);
  reset (in_file,1);
  writeln;
  writeln('Place a floppy in drive ',copy(out_name,1,2),
              ' to receive ',in_name,' .');
      writeln('Press Enter when ready.');
      readln;
  repeat {until recs_in = 0}
    writeln('Copying ',in_name,' to ',out_name,' ...');

    {save space for log_file}
    assign(log_file,log_name);
    rewrite(log_file);
    writeln(log_file,total_recs); {temporary dummy data}
    close(log_file);

    assign (out_file,out_name);
    rewrite(out_file,1);
    repeat {until recs_in=0 or recs_out=0}
      if recs_not_written = 0 then begin
        blockread(in_file,buffer,buffer_size,recs_in);
        total_recs:=total_recs+recs_in;
        end;
      if recs_in > 0 then blockwrite(out_file,buffer,recs_in,recs_out);
      recs_not_written:=recs_in-recs_out;
      until (recs_in=0) or (recs_out=0);
    close(out_file);

    {need more floppies?}
    if recs_in > 0 then begin
      {erase the temp log file}
      assign(log_file,log_name);
      erase(log_file);

      {prompt for more}
      out_name[length(out_name)]:=chr(ord(out_name[length(out_name)])+1);
      writeln;
      writeln('Replace floppy in drive ',copy(out_name,1,2),
              ' with another to allow the copy process to continue.');
      writeln('Press Enter when ready.');
      readln;
      end;
    until recs_in=0;
  close(in_file);

  {update log file}
  assign(log_file,log_name);
  rewrite(log_file);
  writeln(log_file,total_recs);
  close(log_file);
  end.
