program ebcdic_to_ascii;

const
  ebcdic_to_ascii_hex_table : array[0..15] of string[48] = (
    {         0_ 1_ 2_ 3_ 4_ 5_ 6_ 7_ 8_ 9_ A_ B_ C_ D_ E_ F_ }
    { _0 }  ' 20 10 ?? ?? 20 26 2D ?? ?? ?? ?? ?? 7B 7D 5C 30',
    { _1 }  ' 01 11 ?? ?? ?? ?? 2F ?? 61 6A 7E ?? 41 4A ?? 31',
    { _2 }  ' 02 12 ?? 16 ?? ?? ?? ?? 62 6B 73 ?? 42 4B 53 32',
    { _3 }  ' 03 13 ?? ?? ?? ?? ?? ?? 63 6C 74 ?? 43 4C 54 33',
    { _4 }  ' ?? ?? ?? ?? ?? ?? ?? ?? 64 6D 75 ?? 44 4D 55 34',
    { _5 }  ' 09 ?? 0A ?? ?? ?? ?? ?? 65 6E 76 ?? 45 4E 56 35',
    { _6 }  ' ?? 08 17 ?? ?? ?? ?? ?? 66 6F 77 ?? 46 4F 57 36',
    { _7 }  ' 7F ?? 1B 04 ?? ?? ?? ?? 67 70 78 ?? 47 50 58 37',
    { _8 }  ' ?? 18 ?? ?? ?? ?? ?? ?? 68 71 79 ?? 48 51 59 38',
    { _9 }  ' ?? 19 ?? ?? ?? ?? ?? 60 69 72 7A ?? 49 52 5A 39',
    { _A }  ' ?? ?? ?? ?? ?? 21 ?? 3A ?? ?? ?? ?? ?? ?? ?? ??',
    { _B }  ' 0B ?? ?? ?? 2E 24 2C 23 ?? ?? ?? ?? ?? ?? ?? ??',
    { _C }  ' 0C 1C ?? 14 3C 2A 25 40 ?? ?? ?? ?? ?? ?? ?? ??',
    { _D }  ' 0D 1D 05 15 28 29 5F 27 ?? ?? 5B 5D ?? ?? ?? ??',
    { _E }  ' 0E 1E 06 ?? 2B 3B 3E 3D ?? ?? ?? ?? ?? ?? ?? ??',
    { _F }  ' 0F 1F 07 1A 7C 5E 3F 22 ?? ?? ?? ?? ?? ?? ?? ??');

var

  ascii_of : array[0..255] of byte;
  in_name,out_name : string[80];
  in_file,out_file : file;
  buffer           : array [1..16384] of byte;
  recs             : integer;
  ch               : char;
  i                : integer;

function hex_digit_to_byte(hex:char):byte;
begin
  if      hex in ['0' .. '9'] then hex_digit_to_byte := ord(hex) - 48
  else if hex in ['A' .. 'F'] then hex_digit_to_byte := ord(hex) - 55
  else if hex in ['a' .. 'f'] then hex_digit_to_byte := ord(hex) - 87
  else hex_digit_to_byte := 0;
end;

function hex_to_byte(hex:string):byte;
begin
  if hex = '??' then hex_to_byte := ord('_')
  else hex_to_byte := hex_digit_to_byte(hex[1])*16 + hex_digit_to_byte(hex[2]);
end;

function exists(var filename_of_any_string_length):boolean;
var
  filename : string[255] absolute filename_of_any_string_length;
  filein : text;
begin
  assign(filein,filename);
  {$I-}
  reset(filein);
  {$I+}
  if ioresult = 0 then begin
    exists:=true;
    close(filein);
    end
  else exists:=false;
  end; {exists}

begin

  for i := 0 to 255 do begin
    ascii_of[i] := hex_to_byte(
        copy(ebcdic_to_ascii_hex_table[i mod 16], (i div 16)*3+2, 2));
  end;

  if paramstr(1) = '' then begin
    write ('EBCDIC file name to be converted to ASCII : ');
    readln (in_name);
  end
  else in_name:=paramstr(1);
  if paramstr(2) = '' then begin
    write ('ASCII file name to create : ');
    readln (out_name);
  end
  else out_name:=paramstr(2);
  if in_name = out_name then begin
    writeln;
    writeln('ERROR: You are attempting to copy a file onto itself.',
            '  Program terminating.');
    halt;
  end;
  if not exists(in_name) then begin
    writeln;
    writeln('ERROR: ',in_name,' does not exist.  Program terminating.');
    halt;
  end;

  assign (in_file,in_name);
  reset (in_file,1);
  assign (out_file,out_name);
  rewrite (out_file,1);
  repeat
    blockread (in_file,buffer,16384,recs);
    for i:= 1 to recs do buffer[i] := ascii_of[buffer[i]]; {translate}
    blockwrite (out_file,buffer,recs);
  until recs=0;
  close (in_file);
  close (out_file);
end.
