program encrypt;
{ Written by Tom Kisko, 9/14/89

  Usage:

    ENCRYPT filename.ext

  Action:

    If filename.ext exists
      ENCRYPT makes filename.ENC and deletes filename.ext
    Else
      ENCRYPT makes filename.ext and deletes filename.ENC
}

uses crt;

var
   name,in_name,out_name : string[80];
   in_file,out_file : file;
   buffer           : array [1..16384] of byte;
   recs             : integer;
   ch               : char;
   i,r              : integer;
   n_blks           : integer;
   password,password2 : string;

function exists(var filename_of_any_string_length):boolean;
var
  filename : string[255] absolute filename_of_any_string_length;
  filein : text;
begin
  assign(filein,filename);
  {$I-}
  reset(filein);
  {$I+}
  if ioresult = 0 then begin
    exists:=true;
    close(filein);
  end
  else exists:=false;
end; {exists}

function enc_name(name:string):string;
begin
  if pos('.',name) = 0 then enc_name := name + '.ENC'
  else enc_name := copy(name,1,pos('.',name)-1)+'.ENC';
end;

begin {main}

  {get primary file name}
  if paramstr(1) = '' then begin
    writeln;
    writeln('Usage:');
    writeln;
    writeln('  ENCRYPT filename.ext');
    writeln;
    writeln('Action:');
    writeln;
    writeln('  ENCRYPT prompts for a password and then');
    writeln('  if filename.ext exists');
    writeln('    ENCRYPT makes an encrypted file, filename.ENC and then deletes filename.ext');
    writeln('  Else');
    writeln('    it unencrypts filename.ENC to filename.ext and then deletes filename.ENC');
    writeln;
    writeln('WARNING: DO NOT FORGET YOUR PASSWORD!');
    writeln('         RECOVERY WITHOUT IT IS IMPOSSIBLE!');
    WRITELN;
    halt;
  end
  else name:=paramstr(1);
  for i:= 1 to length(name) do name[i] := upcase(name[i]);

  {.ENC is not allowed}
  if pos('.ENC', name) > 0 then begin
    writeln;
    writeln('ERROR: .ENC is a reserved extension for ENCRYPT.');
    halt;
  end;

  {determine in_name and out_name}
  if exists(name) then begin
    in_name  := name;
    out_name := enc_name(name);
    if exists(out_name) then begin
      writeln('ERROR: Both ', name, ' and ', enc_name(name), ' exist.');
      halt;
    end;
  end
  else begin
    in_name  := enc_name(name);
    out_name := name;
    if not exists(in_name) then begin
      writeln('ERROR: Neither ', name, ' nor ', enc_name(name), ' exist.');
      halt;
    end;
  end;
  writeln(in_name, ' ---> ', out_name);

  {determine password and randseed}
  if paramstr(2) = '' then begin
    write('Password : ');
    password := '';
    repeat
      ch := readkey;
      write(' ');
      if (ch = #3) or (ch = #27) then halt;               {Ctrl-C or Esc halts}
      if ch = #0 then ch := readkey;                      {extended keycode}
      if ch = #8 then begin                               {backspace}
        delete(password,length(password),1);
        write(ch,ch);
      end
      else password := password + upcase(ch);
    until ch = #13;
    write('Verify password : ');
    password2 := '';
    repeat
      ch := readkey;
      write(' ');
      if (ch = #3) or (ch = #27) then halt;               {Ctrl-C or Esc halts}
      if ch = #0 then ch := readkey;                      {extended keycode}
      if ch = #8 then begin                               {backspace}
        delete(password2,length(password),1);
        write(ch,ch);
      end
      else password2 := password2 + upcase(ch);
    until ch = #13;
    if password <> password2 then begin
      writeln('Password did not match!');
      halt;
    end;
  end
  else begin
    password := paramstr(2);
    for i:=1 to length(password) do password[i] := upcase(password[i]);
  end;
  for i := 1 to length(password) do randseed := randseed + i*ord(password[i]);
  writeln;

  {open files}
  assign (in_file,in_name);
  reset (in_file,1);
  assign (out_file,out_name);
  rewrite (out_file,1);

  {process files}
  n_blks := 0;
  repeat
    blockread (in_file,buffer,16384,recs);
    inc(n_blks);

    {encrypt/unencrypt buffer}
    for i:=1 to recs do buffer[i]:=buffer[i] xor random(256);

    blockwrite (out_file,buffer,recs);
  until recs=0;

  {close files}
  close (in_file);
  close (out_file);

  {wipein_ file}
  assign (in_file,in_name);
  reset (in_file,1);
  fillchar(buffer,16384,' ');
  for r := 1 to n_blks-1 do blockwrite (in_file,buffer,16384);
  blockwrite (in_file,buffer,recs);

  erase (in_file);

end.
