program merge;

{merges a set of files into a single file with a header line between each.}

uses dos;

const
  buffer_size = 16384;
var
  DirInfo: SearchRec;
  n_found:integer;
  file_spec : string;
  merge_file_name : string;
  in_file,out_file : file;
  buffer : array[1..buffer_size] of char;

function LeadingZero(w : Word) : String;
var
  s : String;
begin
  Str(w:0,s);
  if Length(s) = 1 then s := '0' + s;
  LeadingZero := s;
end;

function next(var p:word):word;
begin
  inc(p);
  next := p;
end;

procedure append_to_merge_file(file_name : string);
var
  i:byte;
  recs : word;
  p : word;
  dt : DateTime; { For Pack/UnpackTime}
  date_time_str : string;
begin

  UnpackTime(DirInfo.time,dt);
  date_time_str := LeadingZero(dt.month) + '/' +
                   LeadingZero(dt.day)  + '/' +
                   LeadingZero(dt.year-1900)  + ' ' +
                   LeadingZero(dt.hour) + ':' +
                   LeadingZero(dt.min)  + ':' +
                   LeadingZero(dt.sec);

  {write file_name header}
  p := 0;
  buffer[next(p)] := #13; {carriage return}
  buffer[next(p)] := #10; {line feed}
  buffer[next(p)] := #13; {carriage return}
  buffer[next(p)] := #10; {line feed}
  buffer[next(p)] := #13; {carriage return}
  buffer[next(p)] := #10; {line feed}
  for i := 1 to 30 - length(file_name) do buffer[next(p)] := '>';
  buffer[next(p)] := ' ';
  for i := 1 to length(file_name) do buffer[next(p)] := file_name[i];
  buffer[next(p)] := ' ';
  buffer[next(p)] := '[';
  for i := 1 to length(date_time_str) do buffer[next(p)] := date_time_str[i];
  buffer[next(p)] := ']';
  buffer[next(p)] := ' ';
  for i := 1 to 20 do buffer[next(p)] := '<';
  buffer[next(p)] := #13; {carriage return}
  buffer[next(p)] := #10; {line feed}
  buffer[next(p)] := #13; {carriage return}
  buffer[next(p)] := #10; {line feed}
  blockwrite (out_file,buffer,p);

  assign (in_file,file_name);
  reset (in_file,1);
  repeat
    blockread (in_file,buffer,buffer_size,recs);
    if (recs > 0) and (buffer[recs] = ^Z) then dec(recs); {get rid of crtl-z's}
    blockwrite (out_file,buffer,recs);
  until recs=0;
  close (in_file);
end;

begin {main}
  if paramcount <> 2 then begin
    writeln;
    writeln('This program merges a set of text files into a single file');
    writeln('with a header line between each.');
    writeln;
    writeln('Syntax: MERGE wild_file_spec merge_file');
    writeln;
    halt;
  end;

  file_spec := paramstr(1);
  merge_file_name := paramstr(2);
  assign(out_file, merge_file_name);
  rewrite(out_file,1);

  n_found := 0;
  findfirst(file_spec,anyfile,dirinfo);
  while DOSError = 0 do begin
    if (DirInfo.Attr <> Directory) then begin
      inc(n_found);
      append_to_merge_file(DirInfo.Name);
      findnext (dirinfo);
    end;
  end;
  close(out_file);
  writeln(n_found, ' files merged');
end.
