program typewriter;

type
  str80=string[80];

const
  max_prev_line = 10;
  prev_line:array[1..max_prev_line] of str80 = ('','','','','','','','','','');
  cpl:integer = 0; {current prev_line index}
  npl:integer = 0; {number of prev_line's}

var
  line:str80;
  exit_pgm:boolean;

PROCEDURE HELP_RTN;

BEGIN { HELP_RTN }
   clrscr;
   WRITELN;
   TEXTCOLOR(WHITE);
   WRITELN ('       HELP SCREEN FOR THE TYPEWRITER PROGRAM');
   TEXTCOLOR(LIGHTGRAY);
   WRITELN;
   WRITELN;
   WRITELN('     KEY NAME          FUNCTION');
   WRITELN;
   WRITELN('     Return            Prints the current line as displayed.');
   WRITELN('     Left Arrow        Moves cursor to the left.');
   WRITELN('     Right Arrow       Moves cursor to the right.');
   WRITELN('     Home              Moves cursor to the beginning of line.');
   WRITELN('     End               Moves cursor to the end of line.');
   WRITELN('     Ins               Toggle between over-type and insert modes.');
   WRITELN('     Del               Deletes character under cursor.');
   WRITELN('     Back Space        Deletes character to the left of cursor.');
   WRITELN('     Esc               Erases the entire line.');
{
   WRITELN('     Up Arrow          Displays previous lines entered.');
   WRITELN('                       (Moves up a stack of up to 10 lines.)');
   WRITELN('     Down Arrow        Moves down the stack of lines.');
}
   WRITELN('     F1                Display this help screen.');
   WRITELN('     F2                Exit the TYPEWRITER program.');
   WRITELN;
   WRITE('Enter lines to be printed below.');
   WRITELN;
   WRITELN;
END; { HELP_RTN }

procedure line_edit(var line:str80;var exit_pgm:boolean);

function inkey: integer;

  { This function uses DOS function call hex 1 or 8 to get a character from the
    keyboard.  This function works similar to turbo read(kbd,ch) statement.
    You do not need to strike the enter key to get a key code.

    Inkey returns the ASCII code for normal ASCII keys and a value greater
    than or equal to 1000 for extended key codes (e.g., F1 returns 1059).}

const
  echo:boolean = false;   {Make true to echo chars to screen}

type
  registers_record_type = record
    case byte of
      1:(ax,bx,cx,dx,bp,si,di,ds,es,flags: integer);
      1:(al,ah,bl,bh,cl,ch,dl,dh: byte);
    end;

var
  registers: registers_record_type;

begin
  with registers do begin
    if echo then ah := $1
    else ah := $8;
    msdos(registers);
    if al = 0 then begin  {If the char is an extended code get the 2nd byte}
      msdos(registers);
      inkey := 1000+al;
      end
    else inkey := al;     {else standard ASCII key code}
    end; {with registers}
 end; {inkey}

var
  a : integer;
  ch : char;
  done:boolean;
  i : integer;
  insert_mode : boolean;
  p : integer; {next ch gets added after this pos in line}

begin {line_edit}
  done:=false;
  line:='';
  p:=0;
  insert_mode:=false;
  while not done do begin
    a:=inkey;
    case a of
      32..126 : begin
        ch:=chr(a);
        write(ch);
        if insert_mode or (p = ord(line[0])) then begin
          for i:=p+1 to ord(line[0]) do write(line[i]);
          for i:=ord(line[0]) downto p+1 do begin
            line[i+1]:=line[i];
            write(chr(8));
            end;
          line[p+1]:=ch;
          line[0]:=chr(ord(line[0])+1);
          end
        else begin {overtype}
          line[p+1]:=ch;
          end;
        p:=p+1;
        end;
      8 : begin {back space}
        if p >  0 then begin
          write(chr(8));
          for i:=p+1 to ord(line[0]) do begin
            write(line[i]);
            line[i-1]:=line[i];
            end;
          write(' ',chr(8));
          for i:=p+1 to ord(line[0]) do write(chr(8));
          p:=p-1;
          line[0]:=chr(ord(line[0])-1);
          end;
        end;
      1083 : begin {del}
        if p < ord(line[0]) then begin
          for i:=p+2 to ord(line[0]) do begin
            write(line[i]);
            line[i-1]:=line[i];
            end;
          write(' ',chr(8));
          for i:=p+2 to ord(line[0]) do write(chr(8));
          line[0]:=chr(ord(line[0])-1);
          end;
        end;
      13 : begin
        done:=true;
        writeln;
        if npl < max_prev_line then begin
          npl:=npl+1;
          end
        else begin
          for i:=2 to npl do prev_line[i-1]:=prev_line[i];
          end;
        prev_line[npl]:=line;
        cpl:=1;
(*
        writeln;
        writeln(lst,line);
        line:='';
        p:=0;
        insert_mode:=false;
*)
        end;
      27 : begin {esc}
        line:='';
        writeln;
        end;
      1075 : begin {left arrow}
        if p > 0 then begin
          p:=p-1;
          write(chr(8));
          end;
        end;
      1077 : begin {right arrow}
        if p < ord(line[0]) then begin
          p:=p+1;
          write(line[p]);
          end;
        end;
      1082 : begin {insert}
        insert_mode:=not insert_mode;
        end;
      1072 : begin {up arrow}
        if npl > 0 then begin
          cpl:=cpl-1;
          if cpl = 0 then cpl:=npl;
          i:=wherey;
          gotoxy(1,i);
          clreol;
          line:=prev_line[cpl];
          write(line);
          p:=ord(line[0]);
          end;
        end;
      1080 : begin {down arrow}
        if npl > 0 then begin
          cpl:=cpl+1;
          if cpl > npl then cpl:=1;
          i:=wherey;
          gotoxy(1,i);
          clreol;
          line:=prev_line[cpl];
          write(line);
          p:=ord(line[0]);
          end;
        end;
      1071 : begin {home}
        while p > 0 do begin
          p:=p-1;
          write(chr(8));
          end;
        end;
      1079 : begin {end}
        while p < ord(line[0]) do begin
          p:=p+1;
          write(line[p]);
          end;
        end;
      1059 : begin
        help_rtn;
        line:='';
        p:=0;
        insert_mode:=false;
        end;
      1060,26 : begin
         exit_pgm:=true;
         done:=true;
         end;
      else begin
        {write(a:5,',',line,p:3);}
        end;
      end;
    end;
  end; {line_edit}

begin {main}
  HELP_RTN;
  exit_pgm:=false;
  repeat
    line_edit(line,exit_pgm);
    if not exit_pgm then writeln(lst,line);
    until exit_pgm;
  end.
